/* SPDX-License-Identifier: (GPL-2.0 OR MIT) */
/*
 * Copyright 2019 Laurent Pinchart <laurent.pinchart@ideasonboard.com>
 */

#ifndef __DT_BINDINGS_DMA_XLNX_ZYNQMP_DPDMA_H__
#define __DT_BINDINGS_DMA_XLNX_ZYNQMP_DPDMA_H__

#define ZYNQMP_DPDMA_VIDEO0		0
#define ZYNQMP_DPDMA_VIDEO1		1
#define ZYNQMP_DPDMA_VIDEO2		2
#define ZYNQMP_DPDMA_GRAPHICS		3
#define ZYNQMP_DPDMA_AUDIO0		4
#define ZYNQMP_DPDMA_AUDIO1		5

#endif /* __DT_BINDINGS_DMA_XLNX_ZYNQMP_DPDMA_H__ */
