/*
 * Copyright (c) 2017 Andreas Färber
 *
 * SPDX-License-Identifier: (GPL-2.0+ OR MIT)
 */
#ifndef DT_BINDINGS_POWER_OWL_S500_POWERGATE_H
#define DT_BINDINGS_POWER_OWL_S500_POWERGATE_H

#define S500_PD_VDE	0
#define S500_PD_VCE_SI	1
#define S500_PD_USB2_1	2
#define S500_PD_CPU2	3
#define S500_PD_CPU3	4
#define S500_PD_DMA	5
#define S500_PD_DS	6
#define S500_PD_USB3	7
#define S500_PD_USB2_0	8

#endif
