/* SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause */
/* Copyright (c) 2020, Linaro Ltd.  */

#ifndef __DT_BINDINGS_DMA_QCOM_GPI_H__
#define __DT_BINDINGS_DMA_QCOM_GPI_H__

#define QCOM_GPI_SPI		1
#define QCOM_GPI_UART		2
#define QCOM_GPI_I2C		3

#endif /* __DT_BINDINGS_DMA_QCOM_GPI_H__ */
