/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Device Tree constants for the Qualcomm Atheros AR803x PHYs
 */

#ifndef _DT_BINDINGS_QCA_AR803X_H
#define _DT_BINDINGS_QCA_AR803X_H

#define AR803X_STRENGTH_FULL		0
#define AR803X_STRENGTH_HALF		1
#define AR803X_STRENGTH_QUARTER		2

#endif
