/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (c) 2023, Qualcomm Innovation Center, Inc. All rights reserved.
 */

#ifndef _DT_BINDINGS_POWER_QCOM_RPMHPD_H
#define _DT_BINDINGS_POWER_QCOM_RPMHPD_H

/* Generic RPMH Power Domain Indexes */
#define RPMHPD_CX               0
#define RPMHPD_CX_AO		1
#define RPMHPD_EBI		2
#define RPMHPD_GFX		3
#define RPMHPD_LCX		4
#define RPMHPD_LMX		5
#define RPMHPD_MMCX		6
#define RPMHPD_MMCX_AO		7
#define RPMHPD_MX		8
#define RPMHPD_MX_AO		9
#define RPMHPD_MXC		10
#define RPMHPD_MXC_AO		11
#define RPMHPD_MSS              12
#define RPMHPD_NSP		13
#define RPMHPD_NSP0             14
#define RPMHPD_NSP1             15
#define RPMHPD_QPHY             16
#define RPMHPD_DDR              17
#define RPMHPD_XO               18

#endif
